/// header.cc -*- c++ -*- Hans-Peter Stoerr Mon Nov 10 2003
/// $Id: header.cc,v 1.4 2003/12/29 18:06:31 stoerr Exp $

#include "header.h"
#include "options.h"
#include "footprint.h"
#include <string.h>
#include <stdlib.h>
#include <iostream>

char eprotmagic1[]="HPS.";
char eprotmagic2[]="ErrorProtect";
char eprotversion[]="(1.0,ecc-1.3.5)";

char* theheader=NULL;

void initheader()
{
  if (NULL!=theheader) return;
  theheader=new char[headerlen];
  for(int i=0; i<headerlen; ++i) theheader[i]=0;
  snprintf(theheader,headerlen, "%s%s%s;",
           eprotmagic1,eprotmagic2,eprotversion);
  addheaderitem("headersize",headerlen);
}

/// add item to header
void addheaderitem(const char* descr, const char* value)
{
  initheader();
  char buf[headerlen+1];
  snprintf(buf,headerlen, "%s%s=%s;", theheader,descr,value);
  strncpy(theheader,buf,headerlen);
}

/// add item to header
void addheaderitem(const char* descr, const long value)
{
  initheader();
  char buf[headerlen+1];
  snprintf(buf,headerlen, "%s%s=%ld;", theheader,descr,value);
  strncpy(theheader,buf,headerlen);
}

/// writes the header to a file
void writeheader(FILE* outfile)
{
  if (NULL == strstr(theheader,";crc="))
    addheaderitem("crc",crc(theheader));
  fprintf(outfile,"%s",theheader);
  for(int i=strlen(theheader); i<headerlen; ++i) fputc(0,outfile);
}

/// reads header from a file. file is positioned on data, aferwards.
char* readheader(FILE* infile)
{
  theheader=new char[headerlen];
  for(int i=0; i<headerlen; ++i) theheader[i]=0;
  rewind(infile);
  int readbytes = fread(theheader, sizeof(char), headerlen, infile);
  if ( ferror(infile) )
    { perror("readheader"); abort(); }
  if (readbytes < headerlen) {
    std::cerr << progname << ": ecc file " << eccfname << " is too short.\n";
    abort();
  }
  char fullmagicetc[128];
  snprintf(fullmagicetc,128, "%s%s%s;headersize=512;",
           eprotmagic1,eprotmagic2,eprotversion);
  if ( strstr(theheader, fullmagicetc) != theheader ) {
    std::cerr << progname << ": error correction info is from a different "
              << "version of the program. I can't handle that so far.\n";
    abort();
  }
  char* crcpos = strrchr(theheader,';'); *crcpos = 0;
  crcpos = strrchr(theheader,';')+1; 
  assert( strstr(crcpos,"crc=") == crcpos );
  *crcpos = 0; crcpos += 4;
  long readcrc = atol(crcpos);
  if ( crc(theheader) != readcrc ) {
    std::cerr << progname << ": crc problem in the eccfile."
              << "I can't handle that so far.\n";
    abort();
  }
  return theheader+strlen(fullmagicetc);
}

