/// options.cc -*- c++ -*- Hans-Peter Stoerr Mon Oct 27 2003
/// $Id: options.cc,v 1.5 2003/12/30 11:02:15 stoerr Exp $

#include "options.h"

#include <stdlib.h>
#include <iostream>
#include <iomanip>

/// default values for variables in options.h 

bool protectmode=false; // true if protection mode, false if correction mode.
char* progname=NULL;            // name of the program, as it was called.


char* protectedfname=NULL;       // the filename that is protected
char* eccfname=NULL;            // the filename that contains protection info
char* correctedfname=NULL;       // when correcting, the file where the
/// corrected protectedfile is written

int hblen=100;                   // number of blocks put into one hyperblock
const int hbcorr=6;             // number of blocks of error protection info
                                // for one hyperblock. Unchangable

int verbose=0;                  // verbosity level

const char* optstring="?vb:cp";

void printhelp(char* const progname)
{
  std::cerr << progname << ": Synopsis:" << std::endl;
  std::cerr << progname << " [-?] [-c] [-p] [-v] [-b blocklen] protectedfile outfile [correctedfile]" << std::endl;
  std::cerr << "-c: correct, -p: protect, -v: verbose" << std::endl;
  abort();
}

void readoptions(int argc, char * const argv[])
{
  progname = basename(argv[0]);
  {
    bool corr = ( NULL != strstr(progname, "correct") );
    bool prot = ( NULL != strstr(progname, "protect") );
    if ( corr ) protectmode=false;
    if ( prot ) protectmode=true;
    if ( corr == prot ) {
      std::cerr << "Warning: " << progname
                << ": name under which the programm was called "
                << "was not recognized. Defaulting to protection mode.\n";
      protectmode=true;
    }
  }
  char optchar;
  while (-1 != (optchar = getopt(argc,argv,optstring)) ) {
    switch(optchar) {
    case ':':
    case '?': printhelp(progname); // printhelp never returns
    case 'p': protectmode=true; break;
    case 'c': protectmode=false; break;
    case 'v': verbose=1; break;
    case 'b': hblen=atoi(optarg); assert(1<=hblen);
      assert(hblen<=249); break;
    }
  }
  if ( protectmode && (optind+2 != argc) )
    printhelp(progname);        // printhelp never returns
  if ( (!protectmode) && (optind+2 != argc) && (optind+3 != argc) )
    printhelp(progname);        // printhelp never returns
  protectedfname = argv[optind++];
  eccfname = argv[optind++];
  if ( optind == argc ) correctedfname = protectedfname;
  else correctedfname = argv[optind++];
  if(verbose) {
    P(protectmode);
    P(protectedfname);
    P(eccfname);
    P(correctedfname);
    P(hblen);
    P(hbcorr);
    P(verbose);
    P(100*hbcorr/hblen);
  }
}
